// AE Export version 4.0
// This version of the exporter generates XML files.
// 08.08.2005 Helge Mathee
// in behalf of Electric-Umbrella GmbH
// 
// Modified 09.2007  Byron Nash
// -------------------------------------
//Usage instructions XSI:

//Select the camera and any nulls, lights, or polygons to export to After Effects.

//Run the script.

//Choose a location to store the XML file

//Run the companion Script: "XSI_XML_import.jsx" in After effects to import the data into After Effects

//Notes: 
// A layer will be created for every polygon in each mesh, use simple objects to speed the export/import.
// The script only works with square pixel images. 
// You may use drop frame time code frame rates(29.97, 59.94, etc...) 
//      The scripts will round the numbers off for comparison purposes only. (XSI and AE come up with slightly different numbers due to internal rounding)



//comp size is determined from the pass properties
var curPass = ActiveProject.ActiveScene.ActivePass;
var compWidth = GetValue("Passes.RenderOptions.ImageWidth");
var compHeight = GetValue("Passes.RenderOptions.ImageHeight");
var pixAspect = GetValue("Passes.RenderOptions.ImagePixelRatio");

logmessage("Image is " + compWidth + " by " + compHeight + " and Pixel Aspect is " + pixAspect);

var text = '';
var tabs = 0;
var useTriangles = false;
if (pixAspect == 1){
	aeExport2(useTriangles);
	
}else{
 XSIUIToolkit.Msgbox( "The Pixel Aspect Ratio in the Scene Render Options should be set to 1.0. \n\n     Square pixel images are required to get correct conversion", 
          siMsgOkOnly, "Error" ) ;


}
function aeExport2(useTriangles){
       text = '';
       tabs = 0;

       if(!selection.count){
               logmessage('Nothing selected',siError);
               return false;
       }

       var cameras = new ActiveXObject('XSI.Collection');
       var lights = new ActiveXObject('XSI.Collection');
       var nulls = new ActiveXObject('XSI.Collection');

       var passes = ActiveProject.ActiveScene.Passes;
       if(passes.count<1){
               logmessage('Fatal error. No Passes present.',siError);
       }

       var frameIn = GetValue('PlayControl.In');
       var frameOut = GetValue('PlayControl.Out');
       var fps = (Math.round(GetValue("PlayControl.Rate")*100)/100);
	   //var fps = GetValue("PlayControl.Rate");
		logmessage("fps " + fps);
       log('<root framein='+frameIn+' frameout='+frameOut+' fps='+fps+'>');
       tabs++;

       var helper = ActiveSceneRoot.AddNull();
       for(var i=0;i<selection.count;i++){
               var progress = XSIUIToolkit.ProgressBar;
               progress.caption = 'exporting '+selection(0)+' ...';

               if(selection(i).type=='camera'){
                       	var oldCam = selection(i);
						logmessage("Selected Camera is " +oldCam.name);
						var newCam = SIDuplicate(oldCam);
						newCam = newCam(0);
						RemoveAllCns(newCam, null);
						CutObj(newCam);
						SIApplyConstraint("Pose", newCam, oldCam, null);
						AddExpr(newCam +".camera.fov", "", null);
						SetExpr(newCam +".camera.fov", oldCam+".camera.fov", null);
						newCam.name = "AE_Output_Camera";
						SetValue(newCam + ".kine.local.rotorder", 5, null);
						logmessage("Camera to export to After Effects will be " +newCam.name);
						log('<camera name='+selection(i).name+'>');
                       tabs++;
                       var cons = newCam.kinematics.constraints.filter('dircns');
                       var constr = false;
                       if(cons){
                               for(var j=0;j<cons.count;j++){
                                       if(cons(j).constraining(0).type=='CameraInterest'){
                                               cons = cons(j);
                                               constr = cons.constraining(0).kinematics.global;
                                               break;
                                       }
                               }
                       }else{
                               cons = null;
                       }

                       progress.maximum = frameOut-frameIn;
                       progress.visible = true;

                       for(var frame=frameIn;frame<=frameOut;frame++){
                               Application.LogMessage("Number of direction constraints on Camera: " + cons.Count);
                               if(cons.Count != 0){
                                       log('<key'+x3d(newCam,frame)+' fov='+round(   compWidth /(2*Math.tan( 0.017453 * (newCam.ActivePrimitive.Parameters('fov').value(frame)/2)))  )+' roll='+round(cons.parameters('roll').value(frame))+' interest='+round(constr.parameters('posx').value(frame))+'|'+round(constr.parameters('posy').value(frame))+'|'+round(constr.parameters('posz').value(frame))+'>');
                               }else{
                                       log('<key'+x3d(newCam,frame)+' fov='+round( compWidth /(2*Math.tan( 0.017453 * (newCam.ActivePrimitive.Parameters('fov').value(frame)/2))) )+'>');
                               }
                               progress.increment();
                       }
                       progress.visible = false;
                       tabs--;
                       log('</camera>');
               }else if(selection(i).type=='light'){
                       log('<light name='+selection(i).name+' type='+selection(i).ActivePrimitive.parameters('type').value+'>');
                       tabs++;

                       progress.maximum = frameOut-frameIn;
                       progress.visible = true;

                       var shader = selection(i).ActivePrimitive.Parameters('LightShader').source;
                       for(var frame=frameIn;frame<=frameOut;frame++){
                               log('<key'+x3d(selection(i),frame)+' color='+shader.parameters('color').parameters('red').value(frame)+'|'+shader.parameters('color').parameters('green').value(frame)+'|'+shader.parameters('color').parameters('blue').value(frame)+' intensity='+shader.parameters('intensity').value(frame)+'>');
                               progress.increment();
                       }
                       progress.visible = false;

                       tabs--;
                       log('</light>');
               }else if(selection(i).type=='polymsh'){

                       var primitive = selection(i).activePrimitive;
                       if(useTriangles)
                               var facetCount = primitive.geometry.triangles.count;
                       else
                               var facetCount = primitive.geometry.facets.count;
                       var hasDeformation = primitive.constructionhistory.filter('',siOperatorFamily).count>4;;

                       var UVs = false;
                       for(var j=0;j<primitive.geometry.clusters.count;j++){
                               var cls = primitive.geometry.clusters(j);
                               for(var k=0;k<cls.properties.count;k++){
                                               if(cls.properties(k).type=='uvspace'){
                                               UVs = cls.properties(k);
                                               break;
                                       }
                               }

                               if(UVs)
                                       break;
                       }

                       progress.maximum = facetCount;
                       progress.visible = true;
                       var thisFrameOut = (hasDeformation?frameOut:frameIn);

                       for(var j=0;j<facetCount;j++){
                               log('<solid name='+selection(i).name+'_poly'+j+'>');
                               tabs++;

                               for(var frame=frameIn;frame<=thisFrameOut;frame++){
                                       if(useTriangles)
                                               var facet = primitive.geometry(frame).triangles(j);
                                       else
                                               var facet = primitive.geometry(frame).facets(j);

                                       var points = facet.points;
                                       var center = XSIMath.CreateVector3();
                                       var kine = selection(i).kinematics.global.transform(frame);

                                       for(var k=0;k<points.count;k++)
                                               center.AddInPlace(points(k).position);
                                       center.ScaleInPlace(1/points.count);

                                       var x = XSIMath.CreateVector3();
                                       var y = XSIMath.CreateVector3();
                                       var z = XSIMath.CreateVector3();
                                       var pos = XSIMath.CreateVector3();
                                       var transform = XSIMath.CreateTransform();
                                       var rotation = XSIMath.CreateRotation();
                                       var matrix = XSIMath.CreateMatrix3();

                                       x.Sub(points(0).position,points(1).position);
                                       y.Sub(points(2).position,points(1).position);
                                       z.Cross(y,x);
                                       y.Cross(z,x     );
                                       x.NormalizeInPlace();
                                       y.NormalizeInPlace();
                                       z.NormalizeInPlace();

                                       matrix.Set(x.x,x.y,x.z,y.x,y.y,y.z,z.x,z.y,z.z);
                                       rotation.SetFromMatrix3(matrix);
                                       transform.SetRotation(rotation);
                                       transform.SetTranslation(center);
                                       helper.kinematics.local.parameters('rotorder').value = 0;
                                       helper.kinematics.local.transform = transform;
                                       helper.kinematics.local.parameters('rotorder').value = 5;

                                       var totalMax = 0;
                                       for(var k=0;k<points.count;k++){
                                               pos = XSIMath.MapWorldPositionToObjectSpace(transform,points(k).position);
                                               if(Math.abs(pos.x)>totalMax)
                                                       totalMax = Math.abs(pos.x);
                                               if(Math.abs(pos.y)>totalMax)
                                                       totalMax = Math.abs(pos.y);
                                       }
                                       var s = "";
                                       for(var k=0;k<points.count;k++){
                                               pos = XSIMath.MapWorldPositionToObjectSpace(transform,points(k).position);
                                               s += ' pnt='+round(pos.x/totalMax)+"|"+round(pos.y/totalMax);
                                       }
                                       var angles = XSIMath.CreateVector3();
                                       center = XSIMath.MapObjectPositionToWorldSpace(kine,center);
                                       angles.x = helper.kinematics.local.parameters('rotx').value;
                                       angles.y = helper.kinematics.local.parameters('roty').value;
                                       angles.z = helper.kinematics.local.parameters('rotz').value;
                                       log('<key frame='+frame+' size='+round(totalMax)+' pos='+round(center.x,5)+'|'+round(center.y)+'|'+round(center.z)+' ori='+round(angles.x,5)+'|'+round(angles.y)+'|'+round(angles.z)+s+">");

                               }

                               tabs--;
                               log('</solid>');
                               progress.increment();
                       }
                       progress.visible = false;

               }else{
                       log('<null name='+selection(i).name+'>');
                       tabs++;

                       progress.maximum = frameOut-frameIn;
                       progress.visible = true;

                       for(var frame=frameIn;frame<=frameOut;frame++){
                               log('<key'+x3d(selection(i),frame)+'>');
                               progress.increment();
                       }
                       progress.visible = false;

                       tabs--;
                       log('</null>');
               }
       }

       tabs--;
       log('</root>');
       DeleteObj(helper);


       var fileBrowser = XSIUIToolkit.FileBrowser;
       fileBrowser.Filter = "XML (*.xml)|*.xml||";
       fileBrowser.ShowSave();
       var fileName = fileBrowser.filepathname;
       if(!fileName)
               return false;

       var fso = new ActiveXObject('Scripting.FileSystemObject');
       var file = fso.CreateTextFile(fileName);
       file.write(text);
       file.close();
       DeleteObj(newCam);
}

function log(s){
       var t = '';
       for(var i=0;i<tabs;i++)
               t+='\t';
       if(!s)
               text += t+'\n';
       else
               text += t+s+'\n';
}

function x3d(obj,frame){
       var kineState = obj.kinematics.local;
       var s = ' frame='+frame;
       if(obj.type=='camera'){
               s+=' pos='+round(kineState.parameters('posx').value(frame))+'|'+round(kineState.parameters('posy').value(frame))+'|'+round(kineState.parameters('posz').value(frame));
               s+=' rot='+round(kineState.parameters('rotx').value(frame))+'|'+round(-kineState.parameters('roty').value(frame))+'|'+round(-kineState.parameters('rotz').value(frame));
       }else{
               s+=' pos='+round(kineState.parameters('posx').value(frame))+'|'+round(kineState.parameters('posy').value(frame))+'|'+round(kineState.parameters('posz').value(frame));
               s+=' rot='+round(kineState.parameters('rotx').value(frame))+'|'+round(kineState.parameters('roty').value(frame))+'|'+round(kineState.parameters('rotz').value(frame));
               s+=' scl='+round(kineState.parameters('sclx').value(frame))+'|'+round(kineState.parameters('scly').value(frame))+'|'+round(kineState.parameters('sclz').value(frame));
       }
       return s;
}

function round(x){
       return Math.floor(x*1000)/1000;
}